#include <vector>
#include <iostream>

using std::vector;

// Najde vsetky pocet-prvkove podmnoziny mnoziny {t_minT, ..., t_maxT},
// dosadi za t_i hodnoty z pola t (t_i = t[i]) a vrati sucet clenov.
// V premennej pocetClenov vrati pocet najdenych podmnozin. Na vypocet sa
// pouziva rekurzivne volanie funkcie.
//
// minT - najmensi index parametra, ktory mozeme pouzit pri vytvarani podmnozin
// maxT - najvacsi index parametra, ktory mozeme pouzit pri vytvarani podmnozin
// pocet - urcuje, kolko prvkove podmnoziny hladame
// t - pole s parametrami
// pocetClenov - sluzi na vratenie poctu najdenych podmnozin
double pomPolarForma(unsigned int minT, unsigned int maxT, unsigned int pocet,
	const vector<double>& t, unsigned int& pocetClenov)
{
	unsigned int tmpPocetClenov;
	double result;

	// Ak hladame 0-prvkove podmnoziny, tak taka je len jedna a hodnota po
	// dosadeni = 1. Terminalna podmienka rekurzie.
	if (pocet == 0) {
		result = 1;
		pocetClenov = 1;
		return result;
	}

	result = 0;
	pocetClenov = 0;

	// Podmnoziny mozeme kombinovat z parametrov minT az maxT. Kedze vsak
	// potrebujeme najst pocet-prvkove podmnoziny, nemozeme pre prvy prvok
	// pouzit prvok s indexom vacsim ako maxT - (pocet - 1), inak by sme
	// uz nemali pre ostatne prvky volne parametre.
	for (unsigned int i = minT; i <= maxT - pocet + 1; i++) {
		// Rekurzivne si nechame vypocitat podmnoziny, ktore pouzivaju len
		// parametre s indexom o 1 vyssim, ako ma parameter ktory sme pouzili
		// v tomto cykle a zaujimaju nas o 1 mensie podmnoziny. V tmpPocetClenov
		// dostaneme pocet podmnozin v rekurzivnom volani.
		result += t.at(i) * pomPolarForma(i + 1, maxT, pocet - 1, t,
			tmpPocetClenov);
		// Zvysime nas pocet clenov o cleny z rekurzivneho volania.
		pocetClenov += tmpPocetClenov;
	}

	return result;
}

// Vypocita hodnotu polarnej formy polynomu poly
// (poly[0] + t*poly[1] + t^2 * poly[2] + ...) pre parametre t
// (t[0] = t_0, t[1] = t_1, ...).
//
// poly - pole s koeficientami polynomu
// t - pole s parametrami
double polarForma(const vector<double>& poly, const vector<double>& t) {
	double result = 0;
	unsigned int komb;

	// Budeme scitovat cleny patriace roznym mocninam. Jednotlive cleny
	// nam vypocita pomocna funkcia pomPolarForma. Funkcia zaroven vrati
	// pocet clenov (pomocou odkazom odovzdavanej premennej komb). Pri sumacii
	// vynasobime vratenu hodnotu koeficientom pri danej mocnine (poly[i])
	// a predelime poctom clenov v pricitovanom clene (komb).
	for (unsigned int i = 0; i < poly.size(); i++) {
		double r = pomPolarForma(0, t.size() - 1, i, t, komb);
		result += poly.at(i) * r / komb;
	}

	return result;
}

int main(int argc, char** argv) {
	vector<double> poly;
	vector<double> t;

	// Skusobny polynom 3. stupna (1 - 8t + 5t^2 + 2t^3)
	poly.push_back(1.0);
	poly.push_back(-8.0);
	poly.push_back(5.0);
	poly.push_back(2.0);

	// Skusobne parametre (indexovane od 0)
	t.push_back(7.0);
	t.push_back(2.0);
	t.push_back(3.0);

	std::cout << "Vysledok: " << polarForma(poly, t) << std::endl;
}
