program polar;

// Najde vsetky pocet-prvkove podmnoziny mnoziny {t_minT, ..., t_maxT},
// dosadi za t_i hodnoty z pola t (t_i = t[i]) a vrati sucet clenov.
// V premennej pocetClenov vrati pocet najdenych podmnozin. Na vypocet sa
// pouziva rekurzivne volanie funkcie.
//
// minT - najmensi index parametra, ktory mozeme pouzit pri vytvarani podmnozin
// maxT - najvacsi index parametra, ktory mozeme pouzit pri vytvarani podmnozin
// pocet - urcuje, kolko prvkove podmnoziny hladame
// t - pole s parametrami
// pocetClenov - sluzi na vratenie poctu najdenych podmnozin
function pomPolarForma(minT, maxT, pocet : cardinal; const t : array of double;
	var pocetClenov : cardinal) : double;
var
	tmpPocetClenov : cardinal;
	i : integer;
begin
	// Ak hladame 0-prvkove podmnoziny, tak taka je len jedna a hodnota po
	// dosadeni = 1. Terminalna podmienka rekurzie.
	if pocet = 0 then begin
		result := 1;
		pocetClenov := 1;
		exit;
	end;

	result := 0;
	pocetClenov := 0;

	// Podmnoziny mozeme kombinovat z parametrov minT az maxT. Kedze vsak
	// potrebujeme najst pocet-prvkove podmnoziny, nemozeme pre prvy prvok
	// pouzit prvok s indexom vacsim ako maxT - (pocet - 1), inak by sme
	// uz nemali pre ostatne prvky volne parametre.
	for i := minT to maxT - pocet + 1 do begin
		// Rekurzivne si nechame vypocitat podmnoziny, ktore pouzivaju len
		// parametre s indexom o 1 vyssim, ako ma parameter ktory sme pouzili
		// v tomto cykle a zaujimaju nas o 1 mensie podmnoziny. V tmpPocetClenov
		// dostaneme pocet podmnozin v rekurzivnom volani.
		result := result + t[i] * pomPolarForma(i + 1, maxT, pocet - 1, t,
			tmpPocetClenov);
		// Zvysime nas pocet clenov o cleny z rekurzivneho volania.
		pocetClenov := pocetClenov + tmpPocetClenov;
	end;
end;

// Vypocita hodnotu polarnej formy polynomu poly
// (poly[0] + t*poly[1] + t^2 * poly[2] + ...) pre parametre t
// (t[0] = t_0, t[1] = t_1, ...).
//
// poly - pole s koeficientami polynomu
// t - pole s parametrami
function polarForma(const poly : array of double; const t : array of double) :
	double;
var
	komb : cardinal;
	i : integer;
	d : double;
begin
	result := 0;

	// Budeme scitovat cleny patriace roznym mocninam. Jednotlive cleny
	// nam vypocita pomocna funkcia pomPolarForma. Funkcia zaroven vrati
	// pocet clenov (pomocou odkazom odovzdavanej premennej komb). Pri sumacii
	// vynasobime vratenu hodnotu koeficientom pri danej mocnine (poly[i])
	// a predelime poctom clenov v pricitovanom clene (komb).
	for i := 0 to high(poly) do begin
		d := pomPolarForma(0, high(t), i, t, komb);
		result += poly[i] * d / komb;
	end;
end;

////////////////////////////////////////////////////////////////////////////////

var
	poly : array of double;
	t : array of double;

begin
	// Skusobny polynom 3. stupna (1 - 8t + 5t^2 + 2t^3)
	setlength(poly, 4);
	poly[0] := 1.0;
	poly[1] := -8.0;
	poly[2] := 5.0;
	poly[3] := 2.0;

	// Skusobne parametre (indexovane od 0)
	setlength(t, 3);
	t[0] := 7.0;
	t[1] := 2.0;
	t[2] := 3.0;

	writeln('Vysledok: ', polarForma(poly, t):5:3);
end.
